@@ -80,6 +80,7 @@ typedef struct notmuch_crypto {
     notmuch_crypto_context_t* gpgctx;
     notmuch_bool_t verify;
     notmuch_bool_t decrypt;
+    const char *gpgpath;
 } notmuch_crypto_t;
 
 typedef struct notmuch_show_params {
@@ -277,6 +278,13 @@ void
 notmuch_config_set_database_path (notmuch_config_t *config,
 				  const char *database_path);
 
+const char *
+notmuch_config_get_crypto_gpg_path (notmuch_config_t *config);
+
+void
+notmuch_config_set_crypto_gpg_path (notmuch_config_t *config,
+				  const char *gpg_path);
+
 const char *
 notmuch_config_get_user_name (notmuch_config_t *config);
 
