@@ -91,4 +91,8 @@ test_expect_equal \
     "$(dirname ${TEST_DIRECTORY})" \
     "$(echo $PATH|cut -f1 -d: | sed -e 's,/test/valgrind/bin$,,')"
 
+test_begin_subtest 'notmuch is compiled with debugging symbols'
+readelf --sections $(which notmuch) | grep \.debug
+test_expect_equal 0 $?
+
 test_done
