@@ -45,6 +45,12 @@ done,*)
 	;;
 esac
 
+# Save STDOUT to fd 6 and STDERR to fd 7.
+exec 6>&1 7>&2
+# Make xtrace debugging (when used) use redirected STDERR, with verbose lead:
+BASH_XTRACEFD=7
+export PS4='+(${BASH_SOURCE}:${LINENO}): ${FUNCNAME[0]:+${FUNCNAME[0]}(): }'
+
 # Keep the original TERM for say_color and test_emacs
 ORIGINAL_TERM=$TERM
 
@@ -67,6 +73,8 @@ if [[ ( -n "$TEST_EMACS" && -z "$TEST_EMACSCLIENT" ) || \
 fi
 TEST_EMACS=${TEST_EMACS:-${EMACS:-emacs}}
 TEST_EMACSCLIENT=${TEST_EMACSCLIENT:-emacsclient}
+TEST_CC=${TEST_CC:-cc}
+TEST_CFLAGS=${TEST_CFLAGS:-"-g -O0"}
 
 # Protect ourselves from common misconfiguration to export
 # CDPATH into the environment
@@ -204,8 +212,6 @@ then
 	print_test_description
 fi
 
-exec 5>&1
-
 test_failure=0
 test_count=0
 test_fixed=0
@@ -225,7 +231,7 @@ die () {
 	then
 		exit $code
 	else
-		exec >&5
+		exec >&6
 		say_color error '%-6s' FATAL
 		echo " $test_subtest_name"
 		echo
@@ -236,7 +242,7 @@ die () {
 
 die_signal () {
 	_die_common
-	echo >&5 "FATAL: $0: interrupted by signal" $((code - 128))
+	echo >&6 "FATAL: $0: interrupted by signal" $((code - 128))
 	exit $code
 }
 
@@ -368,6 +374,8 @@ generate_message ()
 	else
 	    template[subject]="Test message #${gen_msg_cnt}"
 	fi
+    elif [ "${template[subject]}" = "@FORCE_EMPTY" ]; then
+	template[subject]=""
     fi
 
     if [ -z "${template[date]}" ]; then
@@ -544,11 +552,10 @@ test_begin_subtest ()
     fi
     test_subtest_name="$1"
     test_reset_state_
-    # Remember stdout and stderr file descriptors and redirect test
-    # output to the previously prepared file descriptors 3 and 4 (see
-    # below)
+    # Redirect test output to the previously prepared file descriptors
+    # 3 and 4 (see below)
     if test "$verbose" != "t"; then exec 4>test.output 3>&4; fi
-    exec 6>&1 7>&2 >&3 2>&4
+    exec >&3 2>&4
     inside_subtest=t
 }
 
@@ -939,7 +946,7 @@ test_expect_code () {
 test_external () {
 	test "$#" = 4 && { prereq=$1; shift; } || prereq=
 	test "$#" = 3 ||
-	error >&5 "bug in the test script: not 3 or 4 parameters to test_external"
+	error >&6 "bug in the test script: not 3 or 4 parameters to test_external"
 	test_subtest_name="$1"
 	shift
 	test_reset_state_
@@ -1156,6 +1163,19 @@ test_python() {
 		| $cmd -
 }
 
+test_C () {
+    exec_file="test${test_count}"
+    test_file="${exec_file}.c"
+    cat > ${test_file}
+    export LD_LIBRARY_PATH=${TEST_DIRECTORY}/../lib
+    ${TEST_CC} ${TEST_CFLAGS} -I${TEST_DIRECTORY}/../lib -o ${exec_file} ${test_file} -L${TEST_DIRECTORY}/../lib/ -lnotmuch -ltalloc
+    echo "== stdout ==" > OUTPUT.stdout
+    echo "== stderr ==" > OUTPUT.stderr
+    ./${exec_file} "$@" 1>>OUTPUT.stdout 2>>OUTPUT.stderr
+    sed "s,${PWD},CWD,g"  OUTPUT.stdout OUTPUT.stderr > OUTPUT
+}
+
+
 # Creates a script that counts how much time it is executed and calls
 # notmuch.  $notmuch_counter_command is set to the path to the
 # generated script.  Use notmuch_counter_value() function to get the
